
# q-expansion of newform 7644.2.a.n, downloaded from the LMFDB on 22 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7644
weight = 2
poly_data = [-8, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [-1, 0], [0, -1], [0, 0], [2, 0], [1, 0], [0, 0], [-4, 1], [2, -1], [-2, 1], [0, 1], [2, -2], [-4, 2], [4, -3], [4, -1], [2, 1], [8, 0], [-6, 0], [8, -2], [-2, -2], [2, -5], [4, -3], [-4, 3], [4, 1], [-14, -1], [12, -2], [8, -4], [-6, 2], [-2, 4], [2, 3], [-4, 0], [4, -4], [-2, -6], [4, -2], [2, 0], [8, 4], [2, 2], [0, -4], [12, -3], [-4, 8], [-2, -5], [-2, 0], [-10, 2], [6, 0], [18, 0], [16, 0], [4, 1], [0, -5], [16, 4], [-2, 4], [-2, 1], [-22, 2], [2, -1], [0, 2], [12, -6], [-14, 1], [24, 0], [0, 8], [2, 3], [22, 0], [12, 4], [-4, -5], [4, -5], [12, 2], [-10, 0], [10, -2], [4, 6], [2, 7], [-2, 2], [2, 7], [8, 2], [-2, 2], [0, 2], [-10, 0], [4, 6], [0, 4], [-2, 0], [26, -1], [14, 2], [-6, -5], [-8, 2], [10, -2], [30, -4], [6, -2], [-8, 2], [38, -1], [10, 2], [10, -6], [-20, 2], [4, -2], [0, 6], [28, -3], [16, 2], [14, -2], [-4, 0], [-8, 10], [8, -9], [-16, 0], [4, -10], [-14, 2], [32, -1], [2, 10], [0, -12], [-2, -7], [32, 1], [6, -8], [-24, -1], [-16, 11], [6, -5], [-18, -4], [32, -2], [-10, 2], [22, 4], [-4, 0], [-12, 14], [6, 3], [-28, 4], [-12, 2], [-2, -12], [2, 5], [18, -5], [-10, 11], [-28, -6], [-18, 6], [-20, -9], [30, -5], [-26, 4], [32, 6], [8, 6], [-26, 3], [0, 14], [-2, 6], [8, -11], [-2, -9], [20, -5], [14, -3], [-4, -10], [-4, -7], [-20, 10], [-46, -1], [12, 8], [26, -8], [-36, 4], [18, -14], [38, 0], [-4, 4], [-22, 13], [-12, -6], [20, -6], [2, 6], [-2, -2], [40, -2], [-4, 8], [28, -8], [36, -3], [-2, 5], [-16, 0], [36, 7], [-2, 6], [24, -3], [26, 8], [18, -1], [-28, 10], [4, 2], [-2, -8], [0, -11], [24, -4], [-6, 2], [-50, 4], [-20, -8], [-38, 4], [2, 8], [38, -11], [38, -6], [-8, -8], [24, -16], [0, 16], [6, 8], [16, 10], [2, -8], [-12, 8], [-12, -6], [10, 4], [0, 15], [2, -8], [4, 6], [-50, -3], [-28, 9], [6, 8], [8, -3], [-22, -2], [-30, 8], [28, -3], [-28, 11], [-14, 17], [8, 16], [14, 4], [10, 15], [-28, -7], [-28, 14], [-54, 6], [24, -16], [-18, -9], [-6, -6], [44, 3], [-4, 0], [0, 13], [10, -9], [-14, -2], [-44, 10], [-22, 9], [-8, 12], [32, 8], [60, 0], [-4, -1], [-10, 1], [-64, 1], [8, -8], [34, -6], [46, -2], [-2, 11], [24, -8], [-46, 2], [0, 18], [-28, 6], [-6, 17], [36, -12], [-22, -3], [8, 6], [30, 12], [-18, 4], [-36, 8], [-8, 4], [10, -12], [20, -18], [16, 3], [-22, 16], [26, 10], [-50, 9], [8, 10], [22, -2], [-20, -14], [-16, -12], [18, -8], [20, -3], [-20, -16], [-32, 11], [56, -7], [-24, -4], [-14, 5], [6, 14], [40, -6], [-26, 16], [10, 0], [-4, 2], [18, 6], [-14, 9], [-28, 0], [-44, 4], [-42, 7], [20, 13], [6, 21], [-42, 11], [-6, -20], [-48, 2], [4, -14], [-30, -4], [32, -15], [32, 10], [30, -20], [10, 16], [20, -20], [-6, -20], [-64, 7], [-46, 2], [-8, 8], [10, -2], [34, 2], [14, -18], [8, 20], [-24, 6], [36, 10], [-16, 4], [34, 7], [52, 8], [-34, 15], [-38, -7], [10, -2], [-56, 0], [-20, 0], [-70, 5], [-28, -4], [-10, 8], [-24, -3], [22, -21], [-36, 12], [8, 14], [40, 0], [32, 16], [68, -3], [-74, 4], [14, 2], [28, 9], [50, -9], [48, 7], [-38, 8], [-26, 6], [22, -2], [-2, -12], [18, -23], [-24, -12], [-2, 12], [-42, 0], [-76, 3], [-30, -6], [-22, 22], [20, 13], [42, -4], [-54, 13], [-26, 7], [-12, -4], [-10, -18], [36, 6], [8, 4], [-22, 6], [34, 10], [-24, -20], [20, -17], [30, -10], [46, -8], [22, -12], [-42, -3], [-32, 22], [-56, 7], [48, -2], [48, 9], [14, 14], [20, -22], [-14, -20], [48, -6], [38, 2], [62, 8], [0, 14], [-32, 0], [-50, -2], [-22, -18], [38, -16], [-20, 0], [56, 5], [-4, -2], [36, 5], [6, 8], [-22, 10], [8, 10], [54, -8], [-8, 10], [32, -13], [36, -21], [26, 8], [6, -4], [22, -24], [-20, 18], [-2, -17], [-22, -10], [20, -2], [30, -23], [16, -5], [-60, 10], [-26, 17], [-2, 14], [20, -22], [-10, -26], [42, -11], [-72, 8], [10, -3], [12, 16], [18, 6], [6, 20], [-32, 28], [-54, -4], [-28, -1], [2, 16], [-16, -12], [82, 2], [-44, -5], [0, 27], [-16, 4], [22, 2], [16, 0], [-68, 12], [-18, 24], [-32, 25], [2, -22], [60, -16], [62, -10], [10, 7], [-24, 0], [-56, 18], [-56, -13], [62, 2], [-14, 5], [-78, -7], [-4, -2], [-76, -7], [-64, -5], [-8, 20], [2, 14], [-86, 6], [68, -13], [24, -13], [-34, 18], [-26, -1], [62, -15], [4, -16], [30, -15], [80, -8], [-14, 26], [48, -8], [48, -2], [-20, -16], [78, -3], [-22, 16], [-18, -22], [12, -24], [-26, -16], [-36, -17], [-26, 7], [38, -18], [28, -13], [12, 2], [18, -5], [34, 0], [-16, 25], [-56, -15], [34, 12], [4, 8], [50, -8], [38, -6], [-40, 23], [-8, 19], [-16, -1], [-26, 4], [6, 0], [22, -11], [-14, 28], [50, 16]]
